package jp.co.ecosysnetwork.ccloca_app.models;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.universal.EsnDebugLog;

/**
 * 運転状態モデル
 */
public class drive_state {
    //region 列挙体
    /**
     * 運行情報
     */
    public enum OPERATION_STATE {
        /**
         * 運行外
         */
        INACTIVE,
        /**
         * 運行中
         */
        ACTIVE
    }
    //endregion
    //region メンバ変数
    /**
     * 排他制御用オブジェクト
     */
    private final Object _lockObj = new Object();
    /**
     * プリファレンス
     */
    private final SharedPreferences _pref;
    /**
     * プリファレンスエディタ
     */
    private final SharedPreferences.Editor _edit;
    /**
     * 最終運転日(yyyyMMdd形式の文字列)
     */
    private String last_update_course_date;
    /**
     * 起動後、位置情報を1回でも受信したか否か
     */
    private boolean _location_recved_first;
    /**
     * 現在のコースインデックス
     */
    private int _now_course_index;
    /**
     * 運行状態
     */
    private OPERATION_STATE _operation_state;
    /**
     * サーバ通信が最終動作した時間
     */
    private long _lastCommActiveMsec;
    /**
     * 位置情報取得が最終動作した時間
     */
    private long _lastLocationActiveMsec;
    //endregion
    //region 生成と廃棄
    /**
     * コンストラクタ
     */
    @SuppressLint("CommitPrefEdits")
    public drive_state(SharedPreferences pref) {
        _pref = pref;
        _edit = _pref.edit();
    }
    //endregion
    //region 外部公開メソッド
    /**
     * デバッグログ文字列の取得
     * @return デバッグログ文字列
     */
    public String toLogString() {
        String ret = String.format(Locale.JAPAN, " last_update_course_date=%s\n", last_update_course_date) +
                String.format(Locale.JAPAN, " _location_recved_first=%s\n", _location_recved_first) +
                String.format(Locale.JAPAN, " _now_course_index=%d\n", _now_course_index) +
                String.format(Locale.JAPAN, " _operation_state=%d\n", _operation_state.ordinal());
        return ret;
    }
    /**
     * 初期化処理
     */
    public void init() {
        synchronized (_lockObj) {
            last_update_course_date = _pref.getString(MyApplication.PKEYlast_update_course_date, "20000101");
            _location_recved_first = false;
            // 運行状態パース
            try {
                _operation_state = OPERATION_STATE.values()[_pref.getInt(MyApplication.PKEY_OPERATION_STATE, OPERATION_STATE.INACTIVE.ordinal())];
            }
            catch (Exception ex) {
                _operation_state = OPERATION_STATE.INACTIVE;
                EsnDebugLog.outputLog(String.format(Locale.JAPAN, "_operation_state load error %s", ex.toString()), EsnDebugLog.DIV.ERR);
            }
            _lastCommActiveMsec = 0;
            _lastLocationActiveMsec = 0;
        }
    }
    //region 最終運転日(yyyyMMdd形式の文字列)
    /**
     * 最終運転日(yyyyMMdd形式の文字列)の取得
     * @return 現最終運転日(yyyyMMdd形式の文字列)
     */
    public String getLastUpdateCourseDate() {
        synchronized (_lockObj) {
            return last_update_course_date;
        }
    }
    /**
     * 最終運転日(yyyyMMdd形式の文字列)の設定
     * @param val 最終運転日(yyyyMMdd形式の文字列)
     * @param isCommit プリファレンスにコミットするか否か
     */
    public void setLastUpdateCourseDate(String val, boolean isCommit) {
        synchronized (_lockObj) {
            if ((last_update_course_date == null) || (!last_update_course_date.equals(val))) {
                last_update_course_date = val;
                _edit.putString(MyApplication.PKEYlast_update_course_date, last_update_course_date);
                if (isCommit) {
                    _edit.commit();
                }
            }
        }
    }
    //endregion
    //region 起動後、位置情報を1回でも受信したか否か
    /**
     * 起動後、位置情報を1回でも受信したか否かの取得
     * @return 起動後、位置情報を1回でも受信したか否か
     */
    public boolean getLocationRecvedFirst() {
        synchronized (_lockObj) {
            return _location_recved_first;
        }
    }
    /**
     * 起動後、位置情報を1回でも受信したか否かの設定
     * @param val 起動後、位置情報を1回でも受信したか否か
     */
    public void setLocationRecvedFirst(boolean val) {
        synchronized (_lockObj) {
             _location_recved_first = val;
        }
    }
    //endregion
    //region 現在のコースインデックス
    /**
     * 現在のコースインデックスの取得
     * @return 現在のコースインデックス
     */
    public int getNowCourseIndex() {
        synchronized (_lockObj) {
            return _now_course_index;
        }
    }
    /**
     * 現在のコースインデックスの設定
     * @param val 現在のコースインデックス
     * @param isCommit プリファレンスにコミットするか否か
     */
    public void setNowCourseIndex(int val, boolean isCommit) {
        synchronized (_lockObj) {
            if (_now_course_index != val) {
                _now_course_index = val;
                _edit.putInt(MyApplication.PKEY_NOW_COURSE_INDEX, _now_course_index);
                if (isCommit) {
                    _edit.commit();
                }
            }
        }
    }
    //endregion
    //region 運行状態
    /**
     * 運行状態の取得
     * @return 運行状態
     */
    public OPERATION_STATE getOperationState() {
        synchronized (_lockObj) {
            return _operation_state;
        }
    }
    /**
     * 運行状態の設定
     * @param val 運行状態
     * @param isCommit プリファレンスにコミットするか否か
     */
    public void setOperationState(OPERATION_STATE val, boolean isCommit) {
        synchronized (_lockObj) {
            if (_operation_state != val) {
                _operation_state = val;
                _edit.putInt(MyApplication.PKEY_OPERATION_STATE, _operation_state.ordinal());
                if (isCommit) {
                    _edit.commit();
                }
            }
        }
    }
    //endregion
    //region サーバ通信が最終動作した時間
    /**
     * サーバ通信が最終動作した時間の取得
     * @return サーバ通信が最終動作した時間
     */
    public long getLastCommActiveMsec() {
        synchronized (_lockObj) {
            return _lastCommActiveMsec;
        }
    }
    /**
     * サーバ通信が最終動作した時間の設定
     * @param val サーバ通信が最終動作した時間
     */
    public void setLastCommActiveMsec(long val) {
        synchronized (_lockObj) {
            _lastCommActiveMsec = val;
        }
    }
    //endregion
    //region 位置情報取得が最終動作した時間
    /**
     * 位置情報取得が最終動作した時間の取得
     * @return 位置情報取得が最終動作した時間
     */
    public long getLastLocationActiveMsec() {
        synchronized (_lockObj) {
            return _lastLocationActiveMsec;
        }
    }
    /**
     * 位置情報取得が最終動作した時間の設定
     * @param val 位置情報取得が最終動作した時間
     */
    public void setLastLocationActiveMsec(long val) {
        synchronized (_lockObj) {
            _lastLocationActiveMsec = val;
        }
    }
    //endregion
}
